@@ -5,9 +5,10 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 0.9
+;; Version: 0.10
 ;; Package-Requires: ((emacs "27.1") (compat "29.1.4.0"))
 ;; Homepage: https://github.com/minad/org-modern
+;; Keywords: outlines, hypermedia, wp
 
 ;; This file is part of GNU Emacs.
 
@@ -233,7 +234,7 @@ references."
   "Prettify todo statistics."
   :type 'boolean)
 
-(defcustom org-modern-progress '("○" "◔" "◐" "◕" "●")
+(defcustom org-modern-progress '("○" "◔" "◑" "◕" "●")
   "Add a progress indicator to the todo statistics.
 Set to nil to disable the indicator."
   :type '(repeat string))
@@ -250,7 +251,7 @@ You can specify a font `:family'.  The font families `Iosevka', `Hack' and
 `DejaVu Sans' give decent results.")
 
 (defface org-modern-label
-  `((t :height 0.9 :width condensed :weight regular :underline nil))
+  `((t :height 0.8 :width condensed :weight regular :underline nil))
   "Parent face for labels.
 The parameters of this face depend on typographical properties of
 the font and should therefore be adjusted by the user depending
@@ -333,7 +334,6 @@ the font.")
 (defvar-local org-modern--font-lock-keywords nil)
 (defvar-local org-modern--star-cache nil)
 (defvar-local org-modern--hide-stars-cache nil)
-(defvar-local org-modern--reset-hide-leading-stars nil)
 (defvar-local org-modern--checkbox-cache nil)
 (defvar-local org-modern--progress-cache nil)
 (defvar-local org-modern--table-sp-width 0)
@@ -343,8 +343,8 @@ the font.")
 
 (defun org-modern--checkbox ()
   "Prettify checkboxes according to `org-modern-checkbox'."
-  (let ((beg (match-beginning 1))
-        (end (match-end 1)))
+  (let ((beg (match-beginning 3))
+        (end (match-end 3)))
     (put-text-property
      beg end 'display
      (cdr (assq (char-after (1+ beg)) org-modern--checkbox-cache)))))
@@ -482,6 +482,8 @@ the font.")
                                   (nth (logand i 1)
                                        org-modern--hide-stars-cache))))
     (when org-modern-star
+      (when (and (eq org-modern-hide-stars 'leading) org-hide-leading-stars)
+        (put-text-property beg (1+ end) 'face (get-text-property end 'face)))
       (put-text-property
        (if (eq org-modern-hide-stars 'leading) beg end)
        (1+ end) 'display
@@ -657,8 +659,8 @@ the font.")
      `((,(format "^\\*+ +%s " (regexp-opt org-todo-keywords-1 t))
         (0 (org-modern--todo)))))
    (when org-modern-checkbox
-     '(("^[ \t]*\\(?:[-+*]\\|[0-9]+[.)]\\)[ \t]+\\(\\[[ X-]\\]\\)[ \t]"
-        (0 (org-modern--checkbox)))))
+     `((,org-list-full-item-re
+        (3 (org-modern--checkbox) nil t))))
    (when (or org-modern-star org-modern-hide-stars)
      `(("^\\(\\**\\)\\* "
         (0 ,(if (eq org-modern-hide-stars t)
@@ -672,9 +674,6 @@ the font.")
                   org-modern-horizontal-rule)))))
    (when org-modern-table
      '(("^[ \t]*\\(|.*|\\)[ \t]*$" (0 (org-modern--table)))))
-   (when org-modern-tag
-     `((,(concat "^\\*+.*?\\( \\)\\(:\\(?:" org-tag-re ":\\)+\\)[ \t]*$")
-        (0 (org-modern--tag)))))
    (when org-modern-footnote
      `(("^\\(\\[fn:\\)[[:word:]-_]+\\]" ;; Definition
         ,@(if-let ((x (car org-modern-footnote)))
@@ -713,7 +712,17 @@ the font.")
         (1 '(face org-modern-statistics) t)
         (2 ,(if org-modern-progress '(org-modern--progress) ''(face nil display " ")))
         (6 '(face nil display " ")))))
-   '((org-fontify-meta-lines-and-blocks)) ;; Ensure that blocks are properly fontified
+   (when org-modern-tag
+     `((,(concat "^\\*+.*?\\( \\)\\(:\\(?:" org-tag-re ":\\)+\\)[ \t]*$")
+        (0 (org-modern--tag)))))
+   ;; Ensure that blocks are properly fontified, source blocks etc.  This
+   ;; fontification rule must come late such that org-modern does not interfere
+   ;; with source fontification.
+   '((org-fontify-meta-lines-and-blocks))
+   (when org-modern-tag
+     `((,(concat "^[ \t]*#\\+\\(?:filetags\\|FILETAGS\\):\\( +\\)\\(:\\(?:"
+                 org-tag-re ":\\)+\\)[ \t]*$")
+        (0 (org-modern--tag)))))
    (when org-modern-keyword
      `(("^[ \t]*\\(#\\+\\)\\([^: \t\n]+\\):"
         ,@(pcase org-modern-keyword
@@ -751,15 +760,9 @@ the font.")
   "Modern looks for Org."
   :global nil
   :group 'org-modern
-  (when org-modern--reset-hide-leading-stars
-    (setq-local org-hide-leading-stars t
-                org-modern--reset-hide-leading-stars nil))
   (cond
    (org-modern-mode
     (add-to-invisibility-spec 'org-modern)
-    (when (and (eq org-modern-hide-stars 'leading) org-hide-leading-stars)
-      (setq-local org-modern--reset-hide-leading-stars t
-                  org-hide-leading-stars nil))
     (setq
      org-modern--star-cache
      (vconcat (mapcar #'org-modern--symbol org-modern-star))
@@ -822,27 +825,28 @@ the font.")
   (add-hook 'pre-redisplay-functions #'org-modern--pre-redisplay nil 'local)
   (save-excursion
     (save-match-data
-      (when org-modern-todo
-        (goto-char (point-min))
-        (let ((re (format " %s "
-                          (regexp-opt
-                           (append org-todo-keywords-for-agenda
-                                   org-done-keywords-for-agenda) t)))
-              (org-done-keywords org-done-keywords-for-agenda))
-          (while (re-search-forward re nil 'noerror)
-            (org-modern--todo))))
-      (when org-modern-tag
-        (goto-char (point-min))
-        (let ((re (concat "\\( \\)\\(:\\(?:" org-tag-re "::?\\)+\\)[ \t]*$")))
-          (while (re-search-forward re nil 'noerror)
-            (org-modern--tag))))
-      (when org-modern-priority
-        (goto-char (point-min))
-        (while (re-search-forward "\\(\\[#.\\]\\)" nil 'noerror)
-          ;; For some reason the org-agenda-fontify-priorities adds overlays?!
-          (when-let ((ov (overlays-at (match-beginning 0))))
-            (overlay-put (car ov) 'face nil))
-          (org-modern--priority))))))
+      (let (case-fold-search)
+        (when org-modern-todo
+          (goto-char (point-min))
+          (let ((re (format " %s "
+                            (regexp-opt
+                             (append org-todo-keywords-for-agenda
+                                     org-done-keywords-for-agenda) t)))
+                (org-done-keywords org-done-keywords-for-agenda))
+            (while (re-search-forward re nil 'noerror)
+              (org-modern--todo))))
+        (when org-modern-tag
+          (goto-char (point-min))
+          (let ((re (concat "\\( \\)\\(:\\(?:" org-tag-re "::?\\)+\\)[ \t]*$")))
+            (while (re-search-forward re nil 'noerror)
+              (org-modern--tag))))
+        (when org-modern-priority
+          (goto-char (point-min))
+          (while (re-search-forward "\\(\\[#.\\]\\)" nil 'noerror)
+            ;; For some reason the org-agenda-fontify-priorities adds overlays?!
+            (when-let ((ov (overlays-at (match-beginning 0))))
+              (overlay-put (car ov) 'face nil))
+            (org-modern--priority)))))))
 
 ;;;###autoload
 (define-globalized-minor-mode global-org-modern-mode
